//+------------------------------------------------------------------+
//|                                                          FMA.mq5 |
//|                                    Copyright (c) 2019, Marketeer |
//|                          https://www.mql5.com/en/users/marketeer |
//+------------------------------------------------------------------+
#property copyright "2019 (c) Marketeer"
#property link "https://www.mql5.com/en/users/marketeer"
#property version "1.0"
#property description "Future MA\n"

#define BUF_NUM 1

#property indicator_separate_window
#property indicator_buffers BUF_NUM
#property indicator_plots   BUF_NUM

#property indicator_color1 Green
#property indicator_width1 2


#include <IndArray.mqh>
IndicatorArray buffers(BUF_NUM);
IndicatorArrayGetter getter(buffers);



input int Length = 100;
input int _Offset = 0;   // Offset
input int Forecast = 0;


int Offset;
const string ID = "OFFSETTER";


int OnInit()
{
  Offset = _Offset;

  IndicatorSetString(INDICATOR_SHORTNAME, "FMA (" + (string)Length + ")");

  for(int i = 0; i < BUF_NUM; i++)
  {
    PlotIndexSetInteger(i, PLOT_DRAW_TYPE, DRAW_LINE);
    PlotIndexSetInteger(i, PLOT_SHIFT, Forecast);
    PlotIndexSetString(i, PLOT_LABEL, "FMA " + (string)i);
  }
  
  showOffsetter(true);

  return INIT_SUCCEEDED;
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& Time[],
                const double& Open[],
                const double& High[],
                const double& Low[],
                const double& Close[],
                const long& Tick_volume[],
                const long& Volume[],
                const int& Spread[])
{
  int i;

  ArraySetAsSeries(Time, true);
  ArraySetAsSeries(Open, true);

  static datetime lastBar = 0;
  static int barCount = 0;

  if(Time[0] == lastBar && barCount == rates_total && prev_calculated != 0) return rates_total;
  
  Print("Calc ", Offset, " ", prev_calculated, " ", rates_total);

  for(i = 0; i < rates_total; i++)
  {
    for(int k = 0; k < BUF_NUM; k++)
    {
      buffers[k][i] = EMPTY_VALUE;
    }
  }
  
  lastBar = Time[0];
  barCount = rates_total;

  double yy[];
  const int n = Length;
  ArrayResize(yy, n, n + Forecast);

  for(i = 0; i < n; i++)
  {
    yy[i] = Open[i + Offset];
  }

  for(i = 0; i < BUF_NUM; i++)
  {
    buffers[i].set(Offset, yy);
  }

  return rates_total;
}

void showOffsetter(const bool selectable = false)
{
  ObjectCreate(0, ID, OBJ_VLINE, ChartWindowFind(), iTime(_Symbol, _Period, Offset), 0);
  ObjectSetInteger(0, ID, OBJPROP_STYLE, STYLE_DOT);
  ObjectSetInteger(0, ID, OBJPROP_COLOR, clrYellow);
  ObjectSetInteger(0, ID, OBJPROP_BACK, true);
  ObjectSetInteger(0, ID, OBJPROP_SELECTABLE, selectable);
  ObjectSetInteger(0, ID, OBJPROP_ZORDER, 1000);
  ObjectSetString(0, ID, OBJPROP_TEXT, "past <--> future");
}

void OnDeinit(const int)
{
  ObjectDelete(0, ID);
}

void OnChartEvent(const int id, const long &lp, const double &dp, const string &sp)
{
  if(id == CHARTEVENT_OBJECT_DRAG && sp == ID)
  {
    const datetime dt = (datetime)ObjectGetInteger(0, ID, OBJPROP_TIME, 0);
    if(dt > TimeCurrent())
    {
      Offset = 0;
    }
    else
    {
      Offset = iBarShift(_Symbol, _Period, dt);
    }
    ChartSetSymbolPeriod(0, NULL, 0);
    Print("Drag ", Offset);
    
    //
    // MT5 hangs for 15 seconds somewhere after this event/moment and next OnCalculate call
    //
  }
}
